

grammar Results;



options {
  language = Java;
  output=AST;
}

@header{
package org.gcn.gpusimpadaptor.grammar;
import java.util.Map;
import java.util.HashMap;
}

@lexer::header{
package org.gcn.gpusimpadaptor.grammar;
}

@members
{

  double numberStore;
  float time;
  int cycles;
  int programs;
  Map<String, Double> variables;
  
  public void addVariable(String variable, double value){
    if(variables==null)
      variables= new HashMap<String, Double>();
    variables.put(variable, value);
  }
  
  public Map<String, Double> getVariables(){
    return variables;
  }
  
  
  public double getVariable(String variable){
    return variables.get(variable);
  }
  
  public boolean hasVariable(String variable){
    return variables.containsKey(variable);
  }
  
  public void setTime(float time){
    this.time = time;
  }
  
  public float getTime(){
    return time;
  }
  
  public void setCycles(int cycles){
    this.cycles=cycles;
  }
  
  public int getCycles(){
    return cycles;
  }
  
    public void setPrograms(int programs){
    this.programs=programs;
  }
  
  public int getPrograms(){
    return programs;
  }
  
}

protected LETTER: ('A'..'Z')|('a'..'z');
protected FIGURE: ('0'..'9');
ID: LETTER (FIGURE|LETTER)*;
SEMICOLON: ';';
COLON: ':';
INT: (FIGURE)+;
FLOAT: (FIGURE)+ '.' (FIGURE)+('E' ('-')? INT)?;
WS : (' '|'\t'|'\n'|'\r')+ {this.skip();} ;




stat: cyclesline programsline (varline)* timeline;

cyclesline:  'cycles' COLON cycles=INT SEMICOLON{setCycles(Integer.parseInt($cycles.getText()));};
varline :  'var' COLON variable=ID COLON 
            (value=FLOAT {numberStore = Double.parseDouble($value.getText());}
            | '1.#INF00'{numberStore = Double.MAX_VALUE;}
            | '-1.#IND00'{numberStore = Double.MAX_VALUE;}
            | '1.#QNAN0' {numberStore = Double.MAX_VALUE;}
            | 'Infinity'{numberStore = Double.MAX_VALUE;}
            | 'NaN' {numberStore = Double.MAX_VALUE;} )
            SEMICOLON{addVariable($variable.getText(), numberStore);};
timeline:  'time' COLON time=FLOAT 'milliseconds'SEMICOLON{setTime(Float.parseFloat($time.getText()));};
programsline: 'programs' COLON programs=INT SEMICOLON{setPrograms(Integer.parseInt($programs.getText()));};




